Package: TENxIO
Type: Package
Title: Import methods for 10X Genomics files
Version: 1.14.0
Authors@R: c(
    person("Marcel", "Ramos", , "marcel.ramos@sph.cuny.edu", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-3242-0582")),
    person("NCI", role = "fnd",
           comment = c(GrantNo. = "U24CA289073"))
  )
Depends: R (>= 4.5.0), SingleCellExperiment, SummarizedExperiment
Imports: BiocBaseUtils, BiocGenerics, BiocIO, Seqinfo, GenomicRanges,
        HDF5Array, Matrix, MatrixGenerics, methods, readr, rhdf5,
        R.utils, S4Vectors, utils
Suggests: BiocStyle, DropletTestFiles, ExperimentHub, knitr,
        RaggedExperiment (>= 1.33.3), rmarkdown, Rsamtools, tinytest
Description: Provides a structured S4 approach to importing data files
        from the 10X pipelines. It mainly supports Single Cell Multiome
        ATAC + Gene Expression data among other data types. The main
        Bioconductor data representations used are SingleCellExperiment
        and RaggedExperiment.
biocViews: Software, Infrastructure, DataImport, SingleCell
VignetteBuilder: knitr
License: Artistic-2.0
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
BugReports: https://github.com/waldronlab/TENxIO/issues
URL: https://github.com/waldronlab/TENxIO
Collate: 'TENxFile-class.R' 'TENxFileList-class.R'
        'TENxFragments-class.R' 'TENxH5-class.R' 'TENxIO-package.R'
        'TENxMTX-class.R' 'TENxPeaks-class.R' 'TENxTSV-class.R'
        'utils.R'
Date: 2025-12-05
Config/pak/sysreqs: libssl-dev libx11-dev zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:59:13 UTC
RemoteUrl: https://github.com/bioc/TENxIO
RemoteRef: RELEASE_3_23
RemoteSha: 20544bc09f909783c1d3b581838275bafeed6eab
NeedsCompilation: no
Packaged: 2026-04-29 15:07:14 UTC; root
Author: Marcel Ramos [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3242-0582>),
  NCI [fnd] (GrantNo.: U24CA289073)
Maintainer: Marcel Ramos <marcel.ramos@sph.cuny.edu>
Built: R 4.6.0; ; 2026-04-29 15:52:33 UTC; unix
