# Generated by roxygen2: do not edit by hand

export(TENxFile)
export(TENxFileList)
export(TENxFragments)
export(TENxH5)
export(TENxMTX)
export(TENxPeaks)
export(TENxTSV)
exportClasses(TENxFile)
exportClasses(TENxFileList)
exportClasses(TENxFragments)
exportClasses(TENxH5)
exportClasses(TENxMTX)
exportClasses(TENxPeaks)
exportClasses(TENxTSV)
exportMethods("annotation<-")
exportMethods(annotation)
exportMethods(decompress)
exportMethods(dim)
exportMethods(dimnames)
exportMethods(genome)
exportMethods(import)
exportMethods(metadata)
exportMethods(path)
exportMethods(rowData)
exportMethods(rowRanges)
exportMethods(show)
import(SingleCellExperiment)
import(SummarizedExperiment)
importClassesFrom(BiocIO,BiocFile)
importFrom(BiocBaseUtils,checkInstalled)
importFrom(BiocBaseUtils,isScalarCharacter)
importFrom(BiocBaseUtils,selectSome)
importFrom(BiocBaseUtils,setSlots)
importFrom(BiocGenerics,"annotation<-")
importFrom(BiocGenerics,annotation)
importFrom(BiocGenerics,path)
importFrom(BiocIO,decompress)
importFrom(BiocIO,import)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,makeGRangesFromDataFrame)
importFrom(Matrix,readMM)
importFrom(MatrixGenerics,rowRanges)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,isEmpty)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,splitAsList)
importFrom(Seqinfo,"genome<-")
importFrom(Seqinfo,genome)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slotNames)
importFrom(readr,read_tsv)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,untar)
